#!/bin/bash
#
# Copyright (C) 2004-2013 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR
#
# This file is part of le-save-config
#
# le-save-config is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
# USA.

# Dependencies:
# curl, iproute

# Exit codes:
OK=0
NOT_ROOT=1
INEP_NOT_FOUND=2
INVALID_MACADDR=3
ACTION_NOT_FOUND=4
FILE_UNDEFINED=5
NOT_BZ2_FILE=6
FILE_EMPTY=7
UNKNOWN_PARAM=8
CONF_NOT_EXIST=9
PERMISSION_DENIED=10
MALFORMED_URL=11
PROXY_UNRESOLVED=12
HOST_UNRESOLVED=13
CONN_FAILED=14
TIMEOUT=15
POST_ERROR=16

# Proinfodata server
SERVER="http://proinfodata.c3sl.ufpr.br/le-save-config"

function showHelp() {
    echo -e "Usage: $0 (-h) [-udc] <FILE>\n"
    echo    "Upload/download a .bz2 file to/from the LE cloud."
    echo    "If option -u is used, FILE is a local file that will be uploaded, otherwise, if option -d is used,"
    echo -e "FILE is used as the download output file.\n"
    echo    "-h | --help : displays this help message."
    echo    "-u | --upload : upload .bz2 file to server."
    echo    "-d | --download : download .bz2 file from server."
    echo    "-c | --check : check if config exists."
}

if [ $# -eq 0 ]; then
    showHelp
    exit $OK
fi

# Is the script running as root?
if [ "$(id -u)" -ne 0 ]; then
   echo "This script must be run as root." 1>&2
   exit $NOT_ROOT
fi

# Hardware/MAC address
HWADDR=$( (ip addr show eth0 2>/dev/null || ip addr show wlan0 2>/dev/null) | grep "link/ether" | tr -s ' ' | cut -d ' ' -f3 )

# Path for INEP code
INEPPATH="/opt/seed_mec/SEED2/client/conf/inep"
if [ ! -s "$INEPPATH" ]; then
    echo "INEP not found, it should be on file \"$INEPPATH\"." 1>&2
    exit $INEP_NOT_FOUND
fi

# INEP code
INEP=$(cat $INEPPATH)

# Checks if the MAC address is valid
function checkHwAddr() {
    if [ -z $(echo $1 | grep -E "^([0-9a-fA-F]{2}:){5}[0-9a-fA-F]{2}$") ]; then
        echo "Invalid MAC address \"$1\"." 1>&2
        exit $INVALID_MACADDR
    fi
}

# If curl returns an error, then return the appropriate code for it
function getCurlError() {
    # Permission denied to file (INEP code and MAC address don't match)
    if [ "$2" = "You do not have permission to access this file." ]; then
        echo $2
        exit $PERMISSION_DENIED
    fi

    case $1 in
        3 )
            # URL malformed
            exit $MALFORMED_URL
        ;;
        5 )
            # Proxy not resolved
            exit $PROXY_UNRESOLVED
        ;;
        6 )
            # Host not resolved
            exit $HOST_UNRESOLVED
        ;;
        7 )
            # Connection failed
            exit $CONN_FAILED
        ;;
        28 )
            # Timeout
            exit $TIMEOUT
        ;;
        34 )
            # Internal HTTP post error
            exit $POST_ERROR
        ;;
    esac
}

while [ $# -gt 0 ]; do
    case $1 in
        -h | --help )
            showHelp
            exit 0
        ;;

        # OPT 1 -> upload file
        # OPT 2 -> download file
        # OPT 3 -> check if config exists

        -u | --upload )
            OPT="1"
        ;;

        -d | --download )
            OPT="2"
        ;;

        -c | --check )
            OPT="3"
        ;;

        -* | --* )
            echo "Unknown option $1"
            exit $UNKNOWN_PARAM
        ;;

        * )
            if [ -z "$FILE" ]; then
                FILE=$1
            else
                echo "Unknown parameter \"$1\""
                exit $UNKNOWN_PARAM
            fi
        ;;
    esac
    shift
done

# Is the MAC address valid?
checkHwAddr $HWADDR

# Resolve MAC address
HWADDRR=$(echo $HWADDR | sed "s/://g")

# User specified -c (check)
if [ "$OPT" = "3" ]; then
    EXISTS=$(curl -sS "$SERVER/$INEP/$HWADDRR/exists")
    if [ "$EXISTS" = "true" ]; then
        echo "Config exists."
        exit $OK
    elif [ "$EXISTS" = "false" ]; then
        echo "Config doesn't exist."
        exit $CONF_NOT_EXIST
    else
        echo $EXISTS
        exit $PERMISSION_DENIED
    fi
fi

# User didn't specified -u (upload) or -d (download)
if [ -z "$OPT" ]; then
    echo -e "No action found"
    showHelp
    exit $ACTION_NOT_FOUND
fi

# User didn't specified FILE
if [ -z "$FILE" ]; then
    echo "File undefined, use \"$0 --help\" for command syntax." 1>&2
    exit $FILE_UNDEFINED
fi

if [ "$OPT" = "1" ]; then
    # Allow only .bz2 files to be downloaded/uploaded
    if [ "$(file -b $FILE | cut -d ' ' -f1)" != "bzip2" ]; then
        echo "You can only upload .bz2 files." 1>&2
        exit $NOT_BZ2_FILE
    fi

    # Does the file to upload exists and isn't empty?
    if [ ! -s "$FILE" ]; then
        echo "File \"$FILE\" doesn't exist or is empty." 1>&2
        exit $FILE_EMPTY
    fi

    # Upload file (MAC address and INEP are used for authentication)
    echo "Uploading file..."
    RET=$(curl -sSF config=@$FILE "$SERVER/$INEP/$HWADDRR")
    # MUST be used after curl
    getCurlError $? "$RET"

elif [ "$OPT" = "2" ]; then
    # Download file (INEP is used for authentication and finding file)
    echo "Downloading file..."
    curl -sS "$SERVER/$INEP/$HWADDRR" > $FILE
    ret_curl="$?"
    RET="$(cat $FILE)"

    getCurlError $ret_curl "$RET"   
fi
